@@ -37,10 +37,12 @@
 
 typedef struct {
     int64_t filesize;
+    int64_t header_filesize;
     int xing_toc;
     int start_pad;
     int end_pad;
-} MP3Context;
+    int is_cbr;
+} MP3DecContext;
 
 /* mp3 read */
 
@@ -49,7 +51,7 @@ static int mp3_read_probe(AVProbeData *p)
     int max_frames, first_frames = 0;
     int fsize, frames, sample_rate;
     uint32_t header;
-    uint8_t *buf, *buf0, *buf2, *end;
+    const uint8_t *buf, *buf0, *buf2, *end;
     AVCodecContext avctx;
 
     buf0 = p->buf;
@@ -89,7 +91,7 @@ static int mp3_read_probe(AVProbeData *p)
 static void read_xing_toc(AVFormatContext *s, int64_t filesize, int64_t duration)
 {
     int i;
-    MP3Context *mp3 = s->priv_data;
+    MP3DecContext *mp3 = s->priv_data;
 
     if (!filesize &&
         !(filesize = avio_size(s->pb))) {
@@ -113,7 +115,7 @@ static void read_xing_toc(AVFormatContext *s, int64_t filesize, int64_t duration
  */
 static int mp3_parse_vbr_tags(AVFormatContext *s, AVStream *st, int64_t base)
 {
-    MP3Context *mp3 = s->priv_data;
+    MP3DecContext *mp3 = s->priv_data;
     uint32_t v, spf;
     unsigned frames = 0; /* Total number of frames in file */
     unsigned size = 0; /* Total number of bytes in the stream */
@@ -185,12 +187,15 @@ static int mp3_parse_vbr_tags(AVFormatContext *s, AVStream *st, int64_t base)
     if (size && frames && !is_cbr)
         st->codec->bit_rate = av_rescale(size, 8 * c.sample_rate, frames * (int64_t)spf);
 
+    mp3->is_cbr          = is_cbr;
+    mp3->header_filesize = size;
+
     return 0;
 }
 
 static int mp3_read_header(AVFormatContext *s)
 {
-    MP3Context *mp3 = s->priv_data;
+    MP3DecContext *mp3 = s->priv_data;
     AVStream *st;
     int64_t off;
 
@@ -226,7 +231,7 @@ static int mp3_read_header(AVFormatContext *s)
 
 static int mp3_read_packet(AVFormatContext *s, AVPacket *pkt)
 {
-    MP3Context *mp3 = s->priv_data;
+    MP3DecContext *mp3 = s->priv_data;
     int ret, size;
     int64_t pos;
 
@@ -255,52 +260,88 @@ static int mp3_read_packet(AVFormatContext *s, AVPacket *pkt)
     return ret;
 }
 
+static int check(AVFormatContext *s, int64_t pos)
+{
+    int64_t ret = avio_seek(s->pb, pos, SEEK_SET);
+    unsigned header;
+    MPADecodeHeader sd;
+    if (ret < 0)
+        return ret;
+    header = avio_rb32(s->pb);
+    if (ff_mpa_check_header(header) < 0)
+        return -1;
+    if (avpriv_mpegaudio_decode_header(&sd, header) == 1)
+        return -1;
+    return sd.frame_size;
+}
+
 static int mp3_seek(AVFormatContext *s, int stream_index, int64_t timestamp,
                     int flags)
 {
-    MP3Context *mp3 = s->priv_data;
-    AVIndexEntry *ie;
+    MP3DecContext *mp3 = s->priv_data;
+    AVIndexEntry *ie, ie1;
     AVStream *st = s->streams[0];
     int64_t ret  = av_index_search_timestamp(st, timestamp, flags);
-    uint32_t header = 0;
+    int i, j;
+    int dir = (flags&AVSEEK_FLAG_BACKWARD) ? -1 : 1;
 
-    if (!mp3->xing_toc) {
+    if (mp3->is_cbr && st->duration > 0 && mp3->header_filesize > s->data_offset) {
+        int64_t filesize = avio_size(s->pb);
+        int64_t duration;
+        if (filesize <= s->data_offset)
+            filesize = mp3->header_filesize;
+        filesize -= s->data_offset;
+        duration = av_rescale(st->duration, filesize, mp3->header_filesize - s->data_offset);
+        ie = &ie1;
+        timestamp = av_clip64(timestamp, 0, duration);
+        ie->timestamp = timestamp;
+        ie->pos       = av_rescale(timestamp, filesize, duration) + s->data_offset;
+    } else if (mp3->xing_toc) {
+        if (ret < 0)
+            return ret;
+
+        ie = &st->index_entries[ret];
+    } else {
         st->skip_samples = timestamp <= 0 ? mp3->start_pad + 528 + 1 : 0;
 
         return -1;
     }
 
-    if (ret < 0)
-        return ret;
-
-    ie = &st->index_entries[ret];
     ret = avio_seek(s->pb, ie->pos, SEEK_SET);
     if (ret < 0)
         return ret;
 
-    while (!s->pb->eof_reached) {
-        header = (header << 8) + avio_r8(s->pb);
-        if (ff_mpa_check_header(header) >= 0) {
-            ff_update_cur_dts(s, st, ie->timestamp);
-            ret = avio_seek(s->pb, -4, SEEK_CUR);
-
-            st->skip_samples = ie->timestamp <= 0 ? mp3->start_pad + 528 + 1 : 0;
-
-            return (ret >= 0) ? 0 : ret;
+#define MIN_VALID 3
+    for(i=0; i<4096; i++) {
+        int64_t pos = ie->pos + i*dir;
+        for(j=0; j<MIN_VALID; j++) {
+            ret = check(s, pos);
+            if(ret < 0)
+                break;
+            pos += ret;
         }
+        if(j==MIN_VALID)
+            break;
     }
+    if(j!=MIN_VALID)
+        i=0;
 
-    return AVERROR_EOF;
+    ret = avio_seek(s->pb, ie->pos + i*dir, SEEK_SET);
+    if (ret < 0)
+        return ret;
+    ff_update_cur_dts(s, st, ie->timestamp);
+    st->skip_samples = ie->timestamp <= 0 ? mp3->start_pad + 528 + 1 : 0;
+    return 0;
 }
 
 AVInputFormat ff_mp3_demuxer = {
     .name           = "mp3",
     .long_name      = NULL_IF_CONFIG_SMALL("MP2/3 (MPEG audio layer 2/3)"),
-    .priv_data_size = sizeof(MP3Context),
     .read_probe     = mp3_read_probe,
     .read_header    = mp3_read_header,
     .read_packet    = mp3_read_packet,
     .read_seek      = mp3_seek,
+    .priv_data_size = sizeof(MP3DecContext),
     .flags          = AVFMT_GENERIC_INDEX,
     .extensions     = "mp2,mp3,m2a", /* XXX: use probe */
 };
